/**vim: ts=4, wrap, tw=80
  *
  *		--- libkvistatwindow.cpp ---
  *
  * This is a plugin for KVirc 2.0 Phoenix. It allows you to see
  * statistics of your IRC sessions - total words said etc.
  * More features are still to come.
  *
  * (c) Krzysztof Godlewski <kristoff@poczta.wprost.pl>
  *
  * 
  * This program is FREE software. You can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your opinion) any later version.
  *
  * This program is distributed in the HOPE that it will be USEFUL,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.

  * You should have received a copy of the GNU General Public License
  * along with this program. If not, write to the Free Software Foundation,
  * Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  **/


#include <qpushbutton.h>
#include <qmessagebox.h>

#include "libkvistatwindow.h"
#include "libkvistatsystraywidget.h"
#include "libkvistat.h"

#include "kvi_locale.h"

extern KviStatController * g_pStatPluginController;

KviStatWindow::KviStatWindow() : QTabDialog(0, "KviStatWindow", false )
{
	KviStr cap;
	cap.sprintf( __tr("Stats since %s"),
		g_pStatPluginController->statData().startDate.ptr());
	setCaption(cap.ptr());
	labels.setAutoDelete(true);

	QVBox *vb = new QVBox(this);
	vb->setMargin(5);
	vb->resize( 290, 270 );

	setFixedSize( vb->width() + 20, vb->height() + 20 );

	int lab_count = 0;
//
// Words / letters tab
	for( ; lab_count < 8; lab_count++ )
	{
		labels.append(new QLabel(vb));
		labels.at(lab_count)->setTextFormat(PlainText);
	}

	addTab(vb, __tr("Words/Letters"));
//
// Chanlist
	vb = new QVBox(this);
	vb->setMargin(5);
	m_chansView = new QListView(vb);

	const char *columns[] = { "Name", "Joins", "Words", "Kicks", "Bans",
		"Topics" };

#define Col_Name		0
#define Col_Joins		1
#define Col_Words		2
#define Col_Kicks		3
#define Col_Bans		4
#define Col_Topics		5

	for( uint i = 0; i < sizeof(columns) / sizeof(columns[0]); i++ )
		m_chansView->addColumn(columns[i]);

	m_chansView->setColumnWidth( Col_Name, 62 );

	QHBox *hb = new QHBox(vb);
	hb->setMargin(5);
	hb->setSpacing(10);

	QPushButton *b = new QPushButton( __tr("Remove channel"), hb );
	connect(b, SIGNAL(clicked()), SLOT(slotRemoveChan()));

	b = new QPushButton( __tr("Remove all"), hb );
	connect( b, SIGNAL(clicked()), g_pStatPluginController,
		SLOT(slotRemoveAllChans()));

	addTab(vb,__tr("Channels"));

//
// Misc
	vb = new QVBox(this);
	vb->setMargin(5);

	for( ; lab_count < 14; lab_count++ )
	{
		labels.append(new QLabel(vb));
		labels.at(lab_count)->setTextFormat(PlainText);
	}

	addTab(vb, __tr( "Misc" ));
	
	setDefaultButton( __tr( "Reset" ) );
	connect( this, SIGNAL(defaultButtonPressed()), g_pStatPluginController,
		SLOT(slotReset()));

	updateStats();
}

KviStatWindow::~KviStatWindow()
{
}

void KviStatWindow::updateStats()
{
	const KviStatData sd = g_pStatPluginController->statData();

//
// Words/Letters
	KviStr s;
	s.sprintf( __tr("Total Words:\t\t\t%d."), sd.totalWords );
	labels.at(0)->setText(s.ptr());
	s.sprintf( __tr("Total Letters:\t\t\t%d."), sd.totalLetters );
	labels.at(1)->setText(s.ptr());
	s.sprintf( __tr("Channels Words:\t\t\t%d."), sd.chansWords );
	labels.at(2)->setText(s.ptr());
	s.sprintf( __tr("Channels Letters:\t\t%d."), sd.chansLetters );
	labels.at(3)->setText(s.ptr());
	s.sprintf( __tr("Queries Words:\t\t\t%d."), sd.queriesWords );
	labels.at(4)->setText(s.ptr());
	s.sprintf( __tr("Queries Letters:\t\t\t%d."), sd.queriesLetters );
	labels.at(5)->setText(s.ptr());
	s.sprintf( __tr("DCCs Words:\t\t\t%d."), sd.dccsWords );
	labels.at(6)->setText(s.ptr());
	s.sprintf( __tr("DCCs Letters:\t\t\t%d."), sd.dccsLetters );
	labels.at(7)->setText(s.ptr());

//
// Misc
	s.sprintf( __tr("Number of IRC sessions:\t\t%d."), sd.onIrc );
	labels.at(8)->setText(s.ptr());
	s.sprintf( __tr("Words Record:\t\t\t%d."), sd.wordsRecord );
	labels.at(9)->setText(s.ptr());
	s.sprintf( __tr("Number of kicks given:\t\t%d."), sd.kicks );
	labels.at(10)->setText(s.ptr());
	s.sprintf( __tr("Number of bans set:\t\t%d."), sd.bans );
	labels.at(11)->setText(s.ptr());
	s.sprintf( __tr("Number of joins to channels:\t%d."), sd.joins );
	labels.at(12)->setText(s.ptr());
	s.sprintf( __tr("Number of topic changes:\t\t%d."), sd.topics );
	labels.at(13)->setText(s.ptr());

//
// Channels
	m_chansView->clear();
	for( const KviStatChan *c = g_pStatPluginController->chanList()->first();
		c; c = g_pStatPluginController->chanList()->next() )
	{
		QListViewItem *vi = new QListViewItem( m_chansView );
		vi->setText(Col_Name, c->name() );
		vi->setText(Col_Words, s.setNum(c->words()).ptr());
		vi->setText(Col_Joins, s.setNum(c->joins()).ptr());
		vi->setText(Col_Kicks, s.setNum(c->kicks()).ptr());
		vi->setText(Col_Bans,  s.setNum(c->bans()).ptr());
		vi->setText(Col_Topics,s.setNum(c->topics()).ptr());
		m_chansView->insertItem(vi);
	}

	m_chansView->setSorting( Col_Words );
}

void KviStatWindow::slotRemoveChan()
{
	if( ! m_chansView->selectedItem() )
		return;

	KviStr cn = m_chansView->selectedItem()->text(Col_Name).latin1();
	if( QMessageBox::warning( 0, __tr("Remove Channel"),
	__tr("Are you sure you want to remove this channel from your statistics ?"),
	__tr("Yes"), __tr("No"), 0, 1) == 0 )	// def. button: No( == 1)
	{
		KviStatChan *chan = g_pStatPluginController->findStatChan(cn.ptr());
		m_chansView->removeItem( m_chansView->selectedItem() );
		g_pStatPluginController->removeChan(chan);
	}
}

#include "libkvistatwindow.moc"
